#include "headers.h"


enum 
    {
    /* display a status bar */
    B,
    /* put the file somewhere other than the current directory */
    O,
    /* silence all output -- the default */
    /* note that if -q is passed explicitly, it will irrevocably override -v and -b */
    Q,
    /* increase the verbosity level. higher levels imply you want the content lower levels would provide */
    /* all verbose output 
is sent to stderr for ease of capture                                          */
    /* 0:  no informational, statistical or debug output                                                 */
    /* 1:  informational output (Sending request FOO to host BAR, Received response header BAZ, etc)     */
    /* 2:  statistical. informational+size of header and body in bytes, round-trip time, etc             */
    /* 3+: debug. statistical+all internal state changes                                                 */
    V
    };


char param[4] = {0};
/* below is to accompany param[O] */
char *outpath; 

int main(int argc, char **argv)
  {
  int i, translen, gotheader;
  int sockd;
  char recvbuf[BUFSIZ+1] = {0};
  char *sendbufp, *offsetp, *errstr;
  FILE *filed;
  struct tls *tlsc;
  uri uristruct;

  i = translen = gotheader = sockd = 0;
  sendbufp = offsetp = 0;
  filed = 0;


  for(i = 0; (i = getopt(argc, argv, "bo:qv")) != -1; i = 0)
    {
    switch(i)
      {
      case 'b':
        param[B] = 1;
      break;

      case 'o':
        if(!param[Q])
          {
          param[O] = 1;
          }
        outpath = optarg;
      break;

      case 'q':
        param[Q] = 1;
        param[B] = 0;
        param[V] = 0;
      break;

    case 'v':
    /* we handle v differently because we want to support different levels of verbosity */
      if(!param[Q])
        {
        param[V]++;
        }
    break;

    default:
      errstr = "apport: uri";
    goto err;
      }
    }


  /* we should probably modify uri_parse to return a zero value on failure... */
  if(uri_parse(argv[optind], &uristruct))
    {
    errstr = "invalid URI.";
    goto err;
    }

  /* if outpath isn't set because we haven't received a -o param, */
  /* then we should set the outpath to the final component of the */
  /* URI.                                                         */
  /* If we can't do that, we should just set it to 'default'.     */
  if(outpath == NULL && !strlen((outpath = (1 + strrchr(uristruct.path, '/')))))
    {
    outpath = "default";
    }

  /* generate our request */
  if( !(sendbufp = reqgen(&uristruct)))
    {
    errstr = "Unknown protocol.";
    goto err;
    }

  if(param[V])
    {
    fprintf(stderr, "request: %s\n", sendbufp);
    }

  /* having a routine or a global variable to track whether we need TLS would be */
  /* nice to add in the future */
  /* Note: at the moment, if the TLS pointer passed is non-NULL, */
  /* dial's return code can only be treated as an indicator of success. */

  if(!strcmp(uristruct.proto, "https"))
    {
    if(! (sockd = dial(uristruct.fqdn, uristruct.proto, &tlsc)))
      {
      errstr = "Couldn't connect to host using the specified protocol.";
      goto err;
      }
    }
  else
    {
    if(! (sockd = dial(uristruct.fqdn, uristruct.proto, NULL)))
      {
      errstr = "Couldn't connect to host using the specified protocol.";
      goto err;
      }
    }

  if(! (filed = fopen(outpath, "w")))
    {
    errstr = "Couldn't open file.";
    goto err;
    }


  if(!strcmp("https", uristruct.proto))
    {
    tls_write(tlsc, sendbufp, strlen(sendbufp));
    }
  else
    {
    if(! (translen = send(sockd, sendbufp, strlen(sendbufp), 0)))
      {
      errstr = "Couldn't transmit request.";
      goto err;
      }
    else if(param[V])
      {
      fprintf(stderr, "sent: %d bytes\n", translen);
      }
    }

  for(gotheader = 0, translen = 1; translen; memset(recvbuf, 0, BUFSIZ+1))
    {
    /* using an abstraction over tls_read/write and send/recv would eliminate */
    /* a lot of redundant code here. */
    if(!strcmp("https", uristruct.proto))
      {
      translen = tls_read(tlsc, recvbuf, BUFSIZ);
      }
    else
      {
      translen = recv(sockd, recvbuf, BUFSIZ, 0);
      }

    if(!translen)
      {
      break;
      }

    if(param[V])
      {
      printf("recv: %d bytes\n", translen);
      }
    
    if(!gotheader)
      {
      offsetp = (strstr(recvbuf, "\r\n\r\n") + 4);
      i = (int) (offsetp - recvbuf);
      gotheader = 1;
      }
    else
      {
      offsetp = recvbuf; 
      i = 0;
      }

    fwrite(offsetp, sizeof(char), translen-i, filed);
    }


  exit(EXIT_SUCCESS);


  err:
  fprintf(stderr, "%s: %s\n", argv[0], errstr);
  exit(EXIT_FAILURE);

  }
